{

Client dataset creator 1.0 by Andrea Raimondi
Copyright (C) 2002 by Andrea Raimondi

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
or go to http://www.gnu.org/licenses/gpl.html

You can reach me via EMail at: rainaple@tin.it or mrpc@supereva.it
Or via normal mail at the following address:

Andrea Raimondi
Via Monte di Dio 66
80132 Napoli
ITALY

}

unit FieldConv;

interface

uses SysUtils, DB;

type
  EFieldConversion = class( Exception );

  ETypeNotSupported = class( EFieldConversion );

  // This is the base conversion class, please note that
  // this class DOESN'T IMPLEMENT all the supported types
  // you can use in this application. Refer to
  // TExtendedFieldConversion class in this same file for
  // details.
  TFieldConversion = class
  private
    procedure SetFieldTypeStr(const Value: String);
  protected
    FFieldTypeStr: String;
    FFieldType: TFieldType;

    procedure AnalyzeDataType( StringType : String );virtual;
    procedure DoAnalysis;
  public
    constructor Create;virtual;
    property FieldTypeStr : String read FFieldTypeStr write SetFieldTypeStr;
    property FieldType : TFieldType read FFieldType;
  end;

  // This is an extension of the conversion class,
  // PLEASE DERIVE FROM HERE to preserve recognization
  // of date/time fields.
  TExtendedFieldConversion = class( TFieldConversion )
  protected
    procedure AnalyzeDataType( StringType : String );override;
  end;

implementation

{ TFieldConversion }

// The following method analyzes the string in the FieldTypeStr property
// and returns the correct ftXXXXX value. If the current type is not supported,
// it will raise a ETypeNotSupported exception.
procedure TFieldConversion.AnalyzeDataType(StringType: String);
var StrPos : Integer;
  // The following routine will check if the string parameter represents a numeric
  // field. If it doesn't, the ResType parameter will assume the ftUnknown value
  // and be passed to other routines to see if it matches any type defined
  procedure NumberedType( AStringType : String;var ResType : TFieldType );
   const NumericTypes : array[ 0..6 ] of String = ( 'SMALLINT','INTEGER','WORD',
                                                    'FLOAT','CURRENCY','BCD',
                                                    'FMTBCD' );
   var NumType_Index : Integer;
       Found : Boolean;
  begin
    Found := False;
    for NumType_Index := 0 to 6 do
    begin
      // Check to see if any number type has been specified
      StrPos := Pos( NumericTypes[ NumType_Index ],AStringType );
      // if it has been specified
      if  StrPos > 0 then
      begin
        Found := True;
        // Set the right type. The choice for the right type
        // is very simple. Since every string in the array
        // has an Index, it's simply needed to remap that
        // index to the correct type. For example, ftSmallInt
        // maps to Index 0 since the array is zero-based and
        // the string 'SMALLINT' is the first string of the
        // array. The same is true in the other conversion
        // routines, also in derived classes.
        case NumType_Index of
          0 : ResType := ftSmallInt;
          1 : ResType := ftInteger;
          2 : ResType := ftWord;
          3 : ResType := ftFloat;
          4 : ResType := ftCurrency;
          5 : ResType := ftBCD;
          6 : ResType := ftFMTBCD;
        end;
      end;
    end;
    // if it has not been specified, pass to the next procedure
    if Not Found then
      ResType := ftUnknown;
  end;

  procedure StringedType( AStringType : String;var ResType : TFieldType );
   const StringTypes : array[ 0..4 ] of String = ( 'STRING','WIDESTRING',
                                                   'FIXEDCHAR','MEMO','FMTMEMO' );
   var StrType_Index : Integer;
       Found : Boolean;
  begin
    Found := False;
    // Check to see if any string type has been specified
      for StrType_Index := 0 to 4 do
      begin
        StrPos := Pos( StringTypes[ StrType_Index ],AStringType );
        // if it has been specified
        if  StrPos > 0 then
        begin
          Found := True;
          // Set the right type
          case StrType_Index of
            0 : ResType := ftString;
            1 : ResType := ftWideString;
            2 : ResType := ftFixedChar;
            3 : ResType := ftMemo;
            4 : ResType := ftFmtMemo;
          end;
        end;
      end;
    // if it has not been specified, pass to the next procedure
    if Not Found then
      ResType := ftUnknown;
  end;

begin
  NumberedType( StringType,FFieldType );
  if FFieldType = ftUnknown then
    StringedType( StringType,FFieldType );
end;

constructor TFieldConversion.Create;
begin
  TObject.Create;
  FFieldType := ftUnknown;
end;

procedure TFieldConversion.DoAnalysis;
begin
  AnalyzeDataType( UpperCase( FFieldTypeStr ) );
end;

procedure TFieldConversion.SetFieldTypeStr(const Value: String);
begin
  FFieldTypeStr := Value;
  DoAnalysis;
end;

{ TExtendedFieldConversion }

procedure TExtendedFieldConversion.AnalyzeDataType(StringType: String);
var StrPos : Integer;
  procedure TimedType( AStringType : String;var ResType : TFieldType );
    const TimeTypes : array[ 0..2 ] of String = ( 'DATE','TIME','DATETIME' );
    var Time_Index : Integer;
        Found : Boolean;
  begin
    for Time_Index := 0 to 2 do
    begin
      StrPos := Pos( TimeTypes[ Time_Index ],StringType );
      if StrPos > 0 then
      begin
        Found := True;
        case Time_Index of
          0 : ResType := ftDate;
          1 : ResType := ftTime;
          2 : ResType := ftDateTime;
        end;
      end;
      if Not Found then
        ResType := ftUnknown;
    end;
  end;

begin
  inherited;
  if FFieldType = ftUnknown then
  begin
    TimedType( StringType,FFieldType );
  end;
end;

end.
